/* #ident "$Id: mini3dconvert.c,v 1.1.1.1 2004/03/30 12:09:31 rzr Exp $" */
/**
 * @author: www.Philippe.COVAL.online.fr - Rev: $Author: rzr $ 
 * Copyright: GPL @ http://rzr.online.fr/licence.htm
 **/
#include "mini3d/mini3d.c"
#include "mini3dexport.c"

//----------------------------------------------------------------------------

int loadScene3dsAsciiNorm( char* filename,  Scene* scene)
{
  int t=0;

  Vertex l,h;
  Vertex r = { 0, 0, 0 };
  Vertex s = { 1,1,1 };
  
  t = loadScene3dsAscii( filename , scene );   
  if ( t < 0 ) return EXIT_FAILURE;
  //debug(". loaded");
  
  statsSceneBox( scene, &l, &h);      
  
  //debug(". transform");
  l = addVV( h, l);
  l = multVf( l, -.5);
  
  r.x = -90;
  
  transformScene( scene, &l, &r ,&s);
  
  scaleSceneInSphere( scene , 1);
  statsSceneBox( scene, &l, &h);

  //l = multVf( l, -1);
  //h = r; 
  //h.y = l.y;
  
  //transformScene( scene, &h, &r ,&s);
  //debug("- transform");
  
  return 0;
}

int usage(int argc, char* argv[])
{
  printf("Usage: mini3dconvert [mode] [export_format] [files] \n");
  printf("Convert 3D mesh from Autodesk 3DS's .ASC to various formats\n");
  printf("Author: Phil COVAL @ http://rzr.online.fr/\n");
  printf("Version: cvs 0.0.2004*\n");

  printf("Options:\nexport_formats :\n--xml | --xmlbin | --vrml1 | --vrml2 | --povray | --langc | --langjava | --ndj \n");
  printf("modes :\n--stdio | --interactive \n");
  printf("Exemples :\n# mini3dconvert --langjava  FILE.asc FILE.java \n");
  printf("# mini3dconvert --stdio --xml  < FILE.asc > FILE.xml \n");
  printf("# mini3dconvert --interactive \n");
  return 0;
}

#define BUFFSIZE 1024

int mainIteractive(int argc, char* argv[])
{
  int t;
  FILE* out = stdout;
  char* filenamein[BUFFSIZE];
  char* filenameout[BUFFSIZE];
  char* mode[BUFFSIZE];
  
  printf("\nInteractive mode\n");

  printf("\ninput file (.asc) ?");
  scanf("%s",filenamein);

  printf("\noutput file ?");
  scanf("%s",filenameout);

  printf("\noutput format ?");
  scanf("%s",mode);

  return saveSceneConvert(filenamein, filenameout, mode);
}


int saveSceneConvert(char* in, char* out, char* mode)
{
  int t;
  FILE* fidout = stdout;

  Scene scene;
  t = loadScene3dsAscii( in , &scene );

  if ( out ) {
    fidout = fopen( out ,"wb");
    if ( ! fidout ) {
      perror("IO err");
      return EXIT_FAILURE;
    }
  }
  if ( t >= 0 ) 
    t = fprintSceneConvert( fidout, &scene, mode);
  freeScene(&scene);
  return t;
}
/// TODO : merge multi object into single object scene
int toObject(Scene* in , Scene* out)
{
  *out = *in;
  
}



int fprintSceneConvert(FILE* out, Scene* arg, char* mode)
{
  Scene oscene;
  Scene * scene = &oscene;
  toObject(arg, scene);

  if ( mode == 0 ) {
    fprintScene( out, scene);
  }  else if ( strcmp("--povray",mode ) == 0)  {
    fprintScenePov( out, scene);
  } else  if ( strcmp("--ndj",mode ) == 0 )  {
    fprintSceneNdj( out, scene);
  } else  if ( strcmp("--vrml1",mode ) == 0 )  {
    fprintSceneVrml1( out, scene);
  } else  if ( strcmp("--vrml2",mode ) == 0 )  {
    fprintSceneVrml2( out, scene);
  } else  if ( strcmp("--langc",mode ) == 0 )  {
    fprintSceneLangC( out, scene);
  } else  if ( strcmp("--langjava",mode ) == 0 )  {
    fprintSceneLangJava( out, scene);
  } else  if ( strcmp("--xml",mode ) == 0 )  {
    fprintSceneXML( out, scene);
  } else  if ( strcmp("--xmlbin",mode ) == 0 )  {
    fprintSceneXMLBin( out, scene);
  } else {
    printf("Format unsuported\n");
    return EXIT_FAILURE;
  }
}



#ifndef LIB
int main(int argc, char* argv[])
{
  int t;
  FILE* out = stdout;
  char* mode = 0;
  
  Scene scene_;
  Vertex l,h;

  //debugf("+ main %d \n",argc);
  //t = loadScene3dsAsciiNorm( argv[1] , &scene_ );   
  if ( argc <= 1 ) {
    usage(argc,argv);
    return EXIT_SUCCESS;
  }

  if ( strcmp( argv[1] , "--stdio"  ) == 0 ) {
    if( argc > 2 && argv[2] ) mode = argv[2];
    t = readScene3dsAscii( stdin, &scene_ );
    t = fprintSceneConvert(out, &scene_,mode);
    freeScene(&scene_);
  } else if ( strcmp(argv[1] , "--int" ) == 0 ) {
    t = mainIteractive(argc,argv);
  } else if ( argc > 3 ) {
    t = saveSceneConvert(argv[2], argv[3] ,argv[1] );
  } else if ( argc > 2 ) {
    t = saveSceneConvert(argv[1], argv[2] , 0 );
  } else if ( argc > 1 ) {
    t = saveSceneConvert(argv[1], 0 , 0 );
  }
  //t = fprintScene(out,  &scene_ );


  if ( t < 0 ) return EXIT_FAILURE;
  //debug(". loaded");
  //statsSceneBox( &scene_, &l, &h);
  //saveScene(&scene_, "/tmp/tmp-obj-3ds-tmp.asc");
  //if ( argc > 3 ) {  }

  
  //saveScenePov(&scene_, "/tmp/tmp-obj-3ds-tmp.pov");
  //system("povray +P /tmp/tmp-obj-3ds-tmp.pov");
  return EXIT_SUCCESS;
}
#endif

