# $Id: mini3dsdl.mk,v 1.1.1.1 2004/03/30 12:09:35 rzr Exp $
#_MAKEFILE_________________________________________[ http://rzr.online.fr ]____
#
# This is a generic makefile for C++ programs
# Author: Philippe_COVAL(a)Yahoo.COM - http://RzR.online.FR - Rev: $Author: rzr $
# Version: 1999.11.29/2002.04.01/d
# Notes: This makefile will probally ONLY run with gnumake
#--------------------------------------------------[ http://rzr.online.fr ]----
CC ?= gcc
CXX ?= g++
LD =g++
AR ?= ar
.SUFFIXES: .c .cpp .cxx .o .fl .inl .h .hpp .hxx .hh .cc
#.SILENT:
# You must define your Architecture here with `uname -s`
#OS = 'shell uname -s' or WIN32, CYGWIN_95-4.0 ...
OS := $(shell uname -s  | sed -e s/\\W/_/g )
VERSION := $(shell g++ --version | sed -e s/\\W/_/g )
# Place for compiled objects
PWD =  $(shell pwd)
INCPATHLIST += .. $(HOME)/share/bin-$(OS)/include
LIBPATHLIST += $(BINDIR) $(HOME)/share/bin-$(OS)/lib

#BINDIR = $(PWD)/../bin-$(OS)-$(MACHTYPE)-${HOSTNAME}-$(USER)/
BINDIR = ${PWD}/
OBJDIR =/tmp/tmp-$(OS)-$(MACHTYPE)-$(HOSTNAME)-$(USER)/$(PWD)/
SRCDIR =./
DIRPATH=$(OBJDIR) $(BINDIR) $(SRCDIR)
# use DIRPATH instead of VPATH
VPATH=$(DIRPATH)			# (./configure overide VPATH)
vpath=$(DIRPATH)
#--------------------------------------------------[ http://rzr.online.fr ]----

INCPATH += $(INCPATHLIST:%=-I%)
LIBPATH += $(LIBPATHLIST:%=-L%)
#--------------------------------------------------[ http://rzr.online.fr ]----
# Put your main program name (warning: using class filename can cause pbs)
# Just put all needed objects, ls *.cpp | sed -e s/.cpp//
SRCS +=$(wildcard $(SRCDIR)*.cpp $(SRCDIR)*.c $(SRCDIR)*.fl $(SRCDIR)*.cxx)
#FILES	=
FILES +=$(SRCS:.c=)
#SRCS=$(FILES:=.cpp)
OBJS=$(FILES:=.o)
PROJECT ?= main
NAME=$(PROJECT)
TARGET = $(NAME)
TARGETSMAIN = 
ifeq ($(STATIC),1)
TARGETSLIBS = lib$(NAME).a
else
TARGETSLIBS = lib$(NAME).so
endif

TARGETSLIBS=
TARGETSMAIN= main mainsdl mainglut  mini3dconvert
OBJS=

default: all

all: ${TARGETSMAIN} 

# vpath != VPATH , and is preserved w/ ./configure
vpath %.cpp %.c %.cc %.cxx %.C %.fl %.h %.hpp %.H %.hxx $(SRCDIR)
vpath %.o %.obj		$(OBJDIR)
vpath % %.exe		$(BINDIR)


-include mkdepend.mk
-include mkdepend-$(OS)-$(VERSION)-$(HOST).mk

#--------------------------------------------------[ http://rzr.online.fr ]----
# Add your own Building directives  here

#--------------------------------------------------[ http://rzr.online.fr ]----
# add your Libs here
#--------------------------------------------------[ http://rzr.online.fr ]----
# Linker
LDLIBS  +=  -lm

# Specific OS libs
ifeq ($(OS),SunOS)
LDLIBS += -lsocket -lnsl -lposix4
else
LDLIBS +=
endif

#--------------------------------------------------[ http://rzr.online.fr ]----
DEFINES += -D$(OS) -D_REENTRANT
#-D_GNU_SOURCE 

# options : -g = debug
#-Wno-implicit -pedantic \
#-Wall -ansi -O3 \
CCFLAGS  += -g
CXXFLAGS += -g
#--------------------------------------------------[ http://rzr.online.fr ]----
# Exec program (main)

lib$(NAME).so: $(OBJS)
	-@mkdir -p $(BINDIR) 2>/dev/null
	cd $(OBJDIR) && \
	$(LD) -shared -o $(BINDIR)$@ $^

lib$(NAME).a: $(OBJS)
	cd $(OBJDIR) &&\
	$(AR) rc $(BINDIR)$@ $^

# Implicit rules / normally this should not be modified
.cpp.o:
	-@mkdir -p $(OBJDIR) 2>/dev/null
	$(CXX) -c -o $(OBJDIR)$(@F) $(CXXFLAGS) $(INCPATH) $(SRCDIR)$<
.c.o:
	-@mkdir -p $(OBJDIR) 2>/dev/null
	$(CC) -c -o $(OBJDIR)$(@F) $(CCFLAGS) $(DEFINES) $(INCPATH) $(SRCDIR)$<
.cxx.o:
	-@mkdir -p $(OBJDIR) 2>/dev/null
	$(CXX) -c -o $(OBJDIR)$(@F) \
	$(CXXFLAGS) $(DEFINES) $(INCPATH) $(SRCDIR)$<

.fl.cxx .fl.h .fl.cpp:
	fluid -c $(SRCDIR)$<
%: %.o
	@mkdir -p ${BINDIR}
	@echo $@ / $^
	cd $(OBJDIR) && \
	$(CXX) -o $(BINDIR)$@ $^ $(LDFLAGS) $(LDLIBS)
#%.o : %.c
#	-@mkdir -p $(OBJDIR) 2>/dev/null
#	$(CXX) -c -o $(OBJDIR)$@ $(CXXFLAGS) $(DEFINES) $(INCPATH) $(SRCDIR)$<

# Dirs
.PHONNY: $(DIRPATH)
$(DIRPATH):
	@-mkdir -p $(DIRPATH) 2> /dev/null
#--------------------------------------------------[ http://rzr.online.fr ]----
# Common Explicit directives
.PHONY: default clean run test exec

run: default exec

exec: $(BINDIR)$(TARGET)
	@cd $(BINDIR)&& \
	time $<

test: clean run clean

clean:
	-@rm 	 ${TARGETSMAIN}  \
	*.o  *~ \#*  *.bak hs~* $(TARGET) a.out $(OBJDIR)*  core \
	>> /dev/null 2>> /dev/null
	-@rm -rf $(OBJDIR)
	@echo "#-$@"

clean-bin:
	@echo ... cleanup your room / or no tv
	rm *.o  *~ \#*  *.bak hs~* $(TARGET) a.out   core; \
	rm -rf $(LIBPATH) $(BINDIR) 
	@echo "#-$@"

clean-all: clean clean-bin
	rm -rf ../bin-* ../tmp-* ../lib-* mkdepend.mk
	@echo "#-$@"

rebuild: clean default


#--------------------------------------------------[ http://rzr.online.fr ]----
# whatever you want : (the right place to add specifics directives)
ls:
	ls $(BINDIR)
xrun:
	xterm -e $(MAINTARGET) &
modes:
	chmod a-x *.cpp *.h


mkdepend-$(OS)-$(HOST).mk: GNUmakefile
	$(CXX) -M $(CXXFLAGS) $(INCPATH) $(SRCS) > $@

mkdepend.mk:GNUmakefile
	$(CXX) -MM $(CXXFLAGS) $(INCPATH) $(SRCS) > $@
info:
	@echo OBJDIR=$(OBJDIR)
	@echo BINDIR=$(BINDIR)
	@echo OBJS=[$(OBJS)]
	@echo OS=$(OS)
	@echo $(SRCS) $(FILES) [$(OBJS)]
	@echo LD_LIBRARY_PATH=$(LD_LIBRARY_PATH)
	@echo DIRPATH=[$(DIRPATH)]
	@echo INCPATH=[$(INCPATH)]
	$(CXX) --version
	gmake --version
	arch
	uname -a
#	env
#	@echo TARGETS=$(TARGETS)
print:
	a2ps $(PRINTFILES)

docxx:
	rm -rf ../doc-api ;\
	doc++ -d ../doc-api -p *.hpp *.h *.H *.hxx


ldlibpath:
	for t in $(LIBPATHLIST)  ; \
	do \
	echo "... adding $$t" ; \
	echo "LD_LIBRARY_PATH=$(LD_LIBRARY_PATH):$$t" ; \
	echo " ...affect"; \
	LD_LIBRARY_PATH=$$LD_LIBRARY_PATH:$$t ; \
	done ; \
	echo " ...check"; \
	echo export LD_LIBRARY_PATH=$$LD_LIBRARY_PATH ; \
	echo "... quit " ;
infos:
	@echo $(THEME) $(VERSION) $(OS)
	$(CXX) --version

love:
	@echo With a computer ? ...really ?

# $Id: mini3dsdl.mk,v 1.1.1.1 2004/03/30 12:09:35 rzr Exp $
#_________________________________________________[ http://rzr.online.fr ]____

mainsdl.o: mainsdl.c
	-@mkdir -p ${OBJDIR}  
	$(CC) -c -o ${OBJDIR}$@  `sdl-config --cflags` $< ${INCPATH}

trace3d.o: trace3d.c
	-@mkdir -p ${OBJDIR}  
	$(CC) -c -o ${OBJDIR}$@  `sdl-config --cflags` $<

mainsdl: mainsdl.o
	-@mkdir -p ${BINDIR}
	cd ${OBJDIR} && \
	$(CC) -o ${BINDIR}${@F}  $(^F) \
	`sdl-config --libs`  -lGL -lm ${LDFLAGS} ${LIBPATH}

mainglut: mainglut.o
	-@mkdir -p ${BINDIR}
	cd ${OBJDIR} && \
	${CC} -o ${BINDIR}${@F}  $(^F) \
	 -lglut -lGL -lm ${LDFLAGS} ${LIBPATH}

main: main.o
	-@mkdir -p ${BINDIR}
	cd ${OBJDIR} && \
	${CC} -o ${BINDIR}${@F}  $(^F) -lm ${LDFLAGS} ${LIBPATH}

mini3dconvert: mini3dconvert.o
	-@mkdir -p ${BINDIR}
	cd ${OBJDIR} && \
	${CC} -o ${BINDIR}${@F}  $(^F) -lm ${LDFLAGS} ${LIBPATH}

debian:
	apt-get install libglut3-dev libglut3

install:
	install -d ${DESTDIR}/usr/local/bin
	install -s mini3dconvert ${DESTDIR}/usr/local/bin/

#_________________________________________________[ http://rzr.online.fr ]____
