/* #ident "$Id: mini3dviewsdl.c,v 1.1.1.1 2004/03/30 12:09:38 rzr Exp $" */
/**
 * @author: www.Philippe.COVAL.online.fr - Rev: $Author: rzr $ 
 * Copyright: GPL @ http://rzr.online.fr/licence.htm
 **/

#include <stdio.h>
#include <stdlib.h>
//#include <unistd.h>

#include <SDL.h>
#include <SDL_endian.h> /* Used for the endian-dependent 24 bpp mode */

#include "mini3d/mini3d.h"
#include "mini3d/mini3d.c"

#define XRES 300
#define YRES 200
#define BPP  32

#include "mini3dviewsdl.h"
#ifndef debugf
#define debugf printf
#define debug(x) debugf("%s\n",x)
#endif



//-----------------------------------------------------------------------------

#ifdef USE_SDL_
void displayObject(SDL_Surface* screen, Object* o)
{
  Uint32 background =0;
  const int w=XRES;
  const int h=YRES;
  float f=0;
  int t=0;
  Point* vPoint =0;
  Vertex center,lo,hi,med;
  debugf("+displayObject %d\n",o->nf);
  
  stats ( o->vv, o->nv , o->vf, o->nf,  &lo, &hi, &med, &center);
  project( &vPoint, o->vv, o->nv, o->vf, o->nf, lo, hi, w, h);
  
  background = SDL_MapRGB( screen->format, 0x11, 0x11, 0x44);
  SDL_FillRect( screen, NULL, background);
  traceSdl ( vPoint,  o->nv,  o->vf,  o->nf );
  SDL_Flip(screen);
  
}



void displaySceneAnim(SDL_Surface* screen, Scene* s)
{
  Uint32 background =0;
  const int w=XRES;
  const int h=YRES;
  float f=0;
  int t=0;
    unsigned int i=0,j=0;
    Point* vPoint =0;
    Vertex center,lo,hi,med;
    Vertex tcenter,tlo,thi,tmed;
    Object* o;
    Vertex rot;
  
    rot.x = .01; rot.y = .02;  rot.z = .03;
    for(j=0;j<100;j++) {
    
      debugf("+displayScene %d\n",s->no);
      o = &(s->vo[0]);
    
      rotateObject(o, rot.x , rot.y , rot.z);
      statsObject( o , &lo, &hi, &med, &center);
      debug(". bounding box");
  
      printV(lo);
      printV(hi);

      for (i=1; i< s->no; i++) {
        o = &(s->vo[i]);
        rotateObject(o, rot.x , rot.y , rot.z);
        statsObject( o , &tlo, &thi, &tmed, &tcenter);
      }
      statsSceneBox(s,&lo,&hi);
      debug(". trace");
      printV(lo);
      printV(hi);
      background = SDL_MapRGB( screen->format, 0x11, 0x11, 0x44);
      SDL_FillRect( screen, NULL, background);
      for (i=0; i< s->no; i++) {
        o = &(s->vo[i]);
        project( &vPoint, o->vv, o->nv, o->vf, o->nf, lo, hi, w, h);
        traceSdl ( vPoint,  o->nv,  o->vf,  o->nf );
      }
      SDL_Flip(screen); 
    }
    //  sleep(3);
}


void displayScene(SDL_Surface* screen, Scene* s)
{
  Uint32 background =0;
  const int w=XRES;
  const int h=YRES;
  float f=0;
  int t=0;
    unsigned int i=0,j=0;
    Point* vPoint =0;
    Vertex center,lo,hi;
    Vertex tcenter,tlo,thi,tmed;
    Object* o;
  
    debugf("+displayScene %d\n",s->no);
    statsSceneBox(s, &lo, &hi);
    background = SDL_MapRGB( screen->format, 0x11, 0x11, 0x44);
    SDL_FillRect( screen, NULL, background);
    if ( SDL_MUSTLOCK(screen) ) {
      if ( SDL_LockSurface(screen) < 0 ) {return;}
    }
    for (i=0; i< s->no; i++) {
      o = &(s->vo[i]);
      project( &vPoint, o->vv, o->nv, o->vf, o->nf, lo, hi, w, h);
      traceSdl ( vPoint,  o->nv,  o->vf,  o->nf );
    }
    if ( SDL_MUSTLOCK(screen) ) { SDL_UnlockSurface(screen); }
    SDL_UpdateRect(screen, 0,0, XRES, YRES);

    SDL_Flip(screen); 
    //  sleep(3);
}

#endif  



int main3dsObj( int argc, char* argv[])
{
  int nFace=0, nVertex=0;
  Object o;

  Vertex center,lo,hi,med;


  int t=0;
  const int w=XRES;
  const int h=YRES;
  float f=0;

  debug("+main3ds");
  //debug( argv[1] );
  t = load3dsAscii( argv[1] , &o);
  statsObject(&o,  &lo, &hi, &med, &center);
  
  displayObject( screen, &o);
  //saveObject(&o,"save-3ds.asc");
#if 0 //def USE_SDL_
  for (t=0; t <= 10*10;t++ )   {
    //rotate( &(o.vv) , o.nv , rot);
    //stats ( o.vv, o.nv , o.vf, o.nf,  &lo, &hi, &med, &center);
    displayObject( screen, &o);
  }
#endif
  freeObject( &o);
  //  sleep(1);
  debug("-main3ds");
  return 0;
    }



/* tranf house

statsSceneBox( &o, &lo, &hi);
rot.x = + (M_PI/2. ) ;
med.x = med.y = med.z = 0.02;
  
transformScene( &o, & lo, &rot, &med );
  
statsSceneBox( &o, &lo, &hi);
lo = multVf(lo,-1);
rot.x =   rot.y =   rot.z = 0;
med.x = med.y = med.z = 1;
  
transformScene( &o, & lo, &rot, &med );
  
statsSceneBox( &o, &lo, &hi);
saveScene( &o, "tmp-3ds.asc");

*/

int main3ds( int argc, char* argv[])
{
  int nFace=0, nVertex=0;
  Scene o;
  
  Vertex center,lo,hi,med = { 1,1,1 };
  Vertex rot = { 0,0,0 };

  int t=0;
  const int w=XRES;
  const int h=YRES;
  float f=0;

  debug("+main3ds");
  //debug( argv[1] );
  t =  loadScene3dsAscii( argv[1] , &o);   assert(t);
  statsSceneBox( &o, &lo, &hi);

#if 0
  rot.x = + (M_PI/2. ) ;
  med.x = med.y = med.z = 1;
  transformScene( &o, & lo, &rot, &med );
  statsSceneBox( &o, &lo, &hi);
#endif

  displayScene( screen, &o);
  sleep(1);
  
  displaySceneAnim( screen, &o);
  debug("-main3ds");
  return 0;
    }

#ifdef USE_SDL_

void DrawPixel(SDL_Surface *screen, int x, int y, Uint32 color)
{

  switch (screen->format->BytesPerPixel) {
  case 1: { /* Assuming 8-bpp */
    Uint8 *bufp;
    bufp = (Uint8 *)screen->pixels + y*screen->pitch + x;
    *bufp = color;
  }
    break;
  case 2: { /* Probably 15-bpp or 16-bpp */
    Uint16 *bufp;
    bufp = (Uint16 *)screen->pixels + y*screen->pitch/2 + x;
    *bufp = color;
  }
    break;
  case 3: { /* Slow 24-bpp mode, usually not used */
    Uint8 *bufp;
    bufp = (Uint8 *)screen->pixels + y*screen->pitch + x * 3;
    if(SDL_BYTEORDER == SDL_LIL_ENDIAN) {
      bufp[0] = color;
      bufp[1] = color >> 8;
      bufp[2] = color >> 16;
    } else {
      bufp[2] = color;
      bufp[1] = color >> 8;
      bufp[0] = color >> 16;
    }
  }
    break;
  case 4: { /* Probably 32-bpp */
    Uint32 *bufp;
    bufp = (Uint32 *)screen->pixels + y*screen->pitch/4 + x;
    *bufp = color;
  }
    break;
  }
}
void DrawPixelRGB(SDL_Surface *screen, int x, int y, Uint8 R, Uint8 G, Uint8 B)
{
  Uint32 color = SDL_MapRGB(screen->format, R, G, B);
  DrawPixel(screen, x,y,color);
}

void DrawPixelNow(SDL_Surface *screen, int x, int y, Uint8 R, Uint8 G, Uint8 B)
{
  if ( SDL_MUSTLOCK(screen) ) {
    if ( SDL_LockSurface(screen) < 0 ) {return;}
  }
  DrawPixelRGB( screen, x,y, R,G,B);
  if ( SDL_MUSTLOCK(screen) ) {
    SDL_UnlockSurface(screen);
  }
  SDL_UpdateRect(screen, x, y, 1, 1);

}

void trace( Point* vPoint, int nVertex,
            Face* vFace, int nFace) 
{
  int i=0;
  Uint32 c=0xFFFFFF;

  float r,g,b,a;
  //debug("+trace");
  for( i=0; i<nFace; i++ ) {
    //debug(vFace[i].material);
    c = getColor(vFace[i].material);
    c = getColorRGBFloat(&r,&g,&b,&a,vFace[i].material);
    //c = getColor( "X");
    //exit(0);
    Line( vPoint[ vFace[i].a ].x, vPoint[ vFace[i].a ].y,
          vPoint[ vFace[i].b ].x, vPoint[ vFace[i].b ].y,
          c);
    Line( vPoint[ vFace[i].c ].x, vPoint[ vFace[i].c ].y,
          vPoint[ vFace[i].b ].x, vPoint[ vFace[i].b ].y,
          c);
    Line( vPoint[ vFace[i].a ].x, vPoint[ vFace[i].a ].y,
          vPoint[ vFace[i].c ].x, vPoint[ vFace[i].c ].y,
          c);
  }
  //debug("-trace");
}

void traceSdl( Point* vPoint, int nVertex,
               Face* vFace, int nFace) 
{
  if ( SDL_MUSTLOCK(screen) ) {
    if ( SDL_LockSurface(screen) < 0 ) {return;}
  }
  trace( vPoint, nVertex, vFace, nFace);
  if ( SDL_MUSTLOCK(screen) ) { SDL_UnlockSurface(screen); }
  SDL_UpdateRect(screen, 0,0, XRES, YRES);
}

#define PutPixel(x,y,c) DrawPixel(screen,x,y,c)



void Line(int x1, int y1, int x2, int y2, Uint32 c)
{
  int dx, dy, sx, sy;
  int count;
  int brc, brmax;
  sx = sy = 1;
  dx = x2 - x1;
  dy = y2 - y1;
  
  if (dx < 0)  {
    dx *= -1;
    sx *= -1;
  }
  if (dy < 0) {
    dy *= -1;
    sy *= -1;
  }
  if (dx > dy) {
    brmax = dx;
    brc = dx / 2;
    PutPixel(x1, y1, c);
    for (count = dx; count; count--) {
      x1 += sx;
      brc += dy;
      if (brc > brmax) {
        brc -= dx;
        y1 += sy;
      }
      PutPixel(x1, y1, c);
    }
  }
  else {
    brmax = dy;
    brc = dy / 2;
    PutPixel(x1, y1, c);
    for (count = dy; count; count--) {
      y1 += sy;
      brc += dx;
      if (brc > brmax)    {
        brc -= dy;
        x1 += sx;
      }
      PutPixel(x1, y1, c);
    }
  }
}

//#define mainSdl  main
int mainSdl( int argc, char* argv[])
{
  debug("+main_sdl");
  SDL_Init( SDL_INIT_VIDEO);
  atexit(SDL_Quit);
  screen = SDL_SetVideoMode( XRES, YRES, BPP, VIDEO_PARAMS);
  if ( screen == NULL ) {
    fprintf(stderr, "Unable to set 640x480 video: %s\n", SDL_GetError());
    exit(1);
  }
  main3ds(argc,argv);  
  sleep(1);
  SDL_Quit(); 
  debug("-main_sdl");
  return( EXIT_SUCCESS);
}


#endif //_h_

//EOF
