/* #ident "$Id: mini3dvieww32.c,v 1.1.1.1 2004/03/30 12:09:38 rzr Exp $" */
/**
 * @author: www.Philippe.COVAL.online.fr - Rev: $Author: rzr $ 
 * Copyright: GPL @ http://rzr.online.fr/licence.htm
 * TODO: argv, argc 4 w32
 **/

#include <windows.h>
#include <GL/gl.h> //check case

#ifndef WINAPI //usefull for mingw32
#define WINAPI 
#endif

#include "mini3d/mini3d.c"
#include "mini3d-tools/mini3dgl.c"


#ifdef CPLUPLUS__
extern "C" {
#endif

static Scene scene_ ;
#ifndef debugf
#define debugf printf
#define debug(x) debugf("%s\n",x)
#endif




LRESULT CALLBACK
WndProc( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam );
  



  /**
   * @author: www.Philippe.COVAL.free.FR - $rev: $author
   * Was need to convert win32 args to ansi way :
   * int argc; char** argv;
   * convertStringWords( GetCommandLine(), &argc, &argv);
   * TODO : check if quotes are wanted or not ?
   **/

  void convertStringWords( char * arg,
                           int *argc, char*** argv);


  void convertStringWords( char * arg,
                           int *argc, char*** argv)
  {
    char *b, *e, *q;
    int i=0;
    int n;
    b = arg;
    q = e  = b ;
    //debug("+ convertStringWords");  debug(arg);
    *argc=0;
    *argv = (char**) malloc( sizeof(char*));

    do {
      while ( *b == ' ' ) b++;
      q=e=b-1;
      do { q = strchr( q + 1 , '"'); }
      while ( (q != NULL) && ( *(q-1) == '\\' ) );

      do { e = strchr( e + 1 , ' '); }
      while ( (e != NULL) && ( *(e-1) == '\\' ) );
      //debugf("%u<%u ?\n",&q,&e);

      if ( (q != NULL) && ( e != NULL) && ( q < e ) ) {
        //debug("quoted");
        do { q = strchr( q + 1 , '"'); }
        while ( (q != NULL) && ( *(q-1) == '\\' ) );
        e = ++q;
      }

      if ( e != NULL) n = (e) - b; else n = strlen(b);
      //debugf("n=%d=%s;\n", n,b);


      *argv = (char**) realloc( *argv, ( (*argc) + 1 ) * sizeof(char*) );
      (*argv)[ *argc ] = (char*) malloc ( n +1);
      strncpy( (*argv)[ *argc ], b , n);
      *( (*argv)[ *argc ] + n ) = '\0';
      //debug( (*argv)[ *argc ] );

      b = e;
      (*argc)++;

    } while ( e != NULL );
    //debug("- convertStringWords");
  }
#ifdef CPLUPLUS__
} //extern "C"
#endif

//------------------------------------------------------------------------------


int WINAPI mini3dinit(int argc, char** argv)
{
  int t=0;
  if ( ( argc < 2 ) ||  ( ! argv[1] ) ) return EXIT_FAILURE;
  t = loadScene3dsAscii( argv[1] , &scene_ );   
  if ( t < 0 ) return EXIT_FAILURE;
  scaleSceneInSphere( &scene_ , 1);
  return EXIT_SUCCESS;
}



#if 1
void WINAPI render() //Scene const * const s)
{
  trace(&scene_);
}

#endif


//------------------------------------------------------------------------------
int WINAPI WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
         LPSTR lpCmdLine, int iCmdShow )
{
  WNDCLASS wc;
  HWND hWnd;
  HDC hDC;
  HGLRC hRC;
  MSG msg;
  BOOL bQuit = FALSE;
  //
  int t=0;
  int argc=0; char** argv = 0;

  //  char* argv0= "cube.asc"; char** argv= new char*[2]; argv[1] = argv0;
  convertStringWords( GetCommandLine(), &argc, &argv);
  if ( argc <= 1 ) {
    printf("Usage: mini3dview32 <3dmesh.asc>\n");
    return EXIT_SUCCESS;
  }
  
  t = mini3dinit(argc, argv);
  if ( t < 0 ) {
    perror("IO Err");
    return EXIT_FAILURE;
  }
  // register window class
  wc.style = CS_OWNDC;
  wc.lpfnWndProc = WndProc;
  wc.cbClsExtra = 0;
  wc.cbWndExtra = 0;
  wc.hInstance = hInstance;
  wc.hIcon = LoadIcon( NULL, IDI_APPLICATION );
  wc.hCursor = LoadCursor( NULL, IDC_ARROW );
  wc.hbrBackground = (HBRUSH)GetStockObject( BLACK_BRUSH );
  wc.lpszMenuName = NULL;
  wc.lpszClassName = "GLSample";
  RegisterClass( &wc );

  // create main window
  hWnd = CreateWindow( "GLSample", "OpenGL Sample",
                       WS_CAPTION | WS_POPUPWINDOW | WS_VISIBLE,
                       0, 0, 256, 256,
                       NULL, NULL, hInstance, NULL );

  // enable OpenGL for the window
  EnableOpenGL( hWnd, &hDC, &hRC );

  while ( !bQuit ) { // program main loop
    // check for messages
    if ( PeekMessage( &msg, NULL, 0, 0, PM_REMOVE ) ) {
      // handle or dispatch messages
      if ( msg.message == WM_QUIT ) {
        bQuit = TRUE;
      } else {
        TranslateMessage( &msg );
        DispatchMessage( &msg );
      }
    } else {
      // OpenGL animation code goes here
      glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
      glClear( GL_COLOR_BUFFER_BIT );
      render();
      SwapBuffers( hDC );
    }
  }
  // shutdown OpenGL
  DisableOpenGL( hWnd, hDC, hRC );
  // destroy the window explicitly
  DestroyWindow( hWnd );
  return msg.wParam;
}

// Window Procedure
LRESULT CALLBACK
WndProc( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam )
{
  switch ( message ) {

  case WM_CREATE:
    return 0;

  case WM_CLOSE:
    PostQuitMessage( 0 );
    return 0;

  case WM_DESTROY:
    return 0;

  case WM_KEYDOWN:
    switch ( wParam ) {
    case VK_ESCAPE:
      PostQuitMessage( 0 );
      return 0;
    }
    return 0;

  default:
    return DefWindowProc( hWnd, message, wParam, lParam );
  }
}

VOID EnableOpenGL( HWND hWnd, HDC * hDC, HGLRC * hRC )
{
  PIXELFORMATDESCRIPTOR pfd;
  int iFormat;

  // get the device context (DC)
  *hDC = GetDC( hWnd );

  // set the pixel format for the DC
  ZeroMemory( &pfd, sizeof( pfd ) );
  pfd.nSize = sizeof( pfd );
  pfd.nVersion = 1;
  pfd.dwFlags = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER;
  pfd.iPixelType = PFD_TYPE_RGBA;
  pfd.cColorBits = 24;
  pfd.cDepthBits = 16;
  pfd.iLayerType = PFD_MAIN_PLANE;
  iFormat = ChoosePixelFormat( *hDC, &pfd );
  SetPixelFormat( *hDC, iFormat, &pfd );

  // create and enable the render context (RC)
  *hRC = wglCreateContext( *hDC );
  wglMakeCurrent( *hDC, *hRC );
}

VOID DisableOpenGL( HWND hWnd, HDC hDC, HGLRC hRC )
{
  wglMakeCurrent( NULL, NULL );
  wglDeleteContext( hRC );
  ReleaseDC( hWnd, hDC );
}

/*

based on // GLSAMPLE.CPP //  by Blaine Hodge

cc-msvc.sh cl mainw32gl.cpp /link user32.lib opengl32.lib gdi32.lib

-
or

mkdir -p ../tmp ../bin
cross.sh g++ -c -o ./mainw32gl.o ./mainw32gl.cpp
cross.sh g++ -o ./mainw32gl.exe ./mainw32gl.o    -lopengl32 -lgdi32
wine mainw32gl.exe


*/
