/* #ident "$Id: parse3ds.c,v 1.1.1.1 2004/03/30 12:09:28 rzr Exp $" */
/**
 * @author: www.Philippe.COVAL.online.fr - Rev: $Author: rzr $ 
 * Copyright: GPL @ http://rzr.online.fr/licence.htm
 **/
#ifndef parse3ds_c_
#define parse3ds_c_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef WANT_MALLOC // might cause troubles on Macosx Project builder
#include <malloc.h>
#endif

#include <math.h>
#include <assert.h>

#include "types3d.h"
#include "parse3ds.h"

#define BSIZE 256
#define BOUND 1000.


enum { trimesh, vertexlist, facelist, vertex, face, material, smoothing };

static char* keyword[] = { "Tri-mesh,", "Vertex list", "Face list",
                           "Vertex", "Face", "Material" , "Smoothing"};

static int nkey= ( smoothing - trimesh + 1); 

#define tag(x) (x - trimesh)

int errorIO(FILE* f,int const t,  const char * msg)
{
  //debug("+ errorIO");
  const char * s = "!!! Err/IO/(exiting)";
  if ( f ) fclose( f );
  if ( msg != 0  ) perror(msg);
  else perror(s);
  //debug("- errorIO");
  exit(t);
  return(t);
}


char* sscanStrQuoted(char* src, char** dest)
{
  char* e=0;
  int n=0;
  //debugf("+ sscanStrQuotted (%s)\n",src);
  //assert( src );
  //assert( dest);
  e = strchr( src, '"');
  if ( e == 0) { *dest=0; return e; }
  src = e;
  //debugf("find 2d quote (%s)\n",e);
  
  do {
    e = strchr( e+1, '"');
  } while ( e && (*(e-1) == '\\') );
  if ( e ==0 ) { *dest=0; return e; }
  n = e - src;
  //debugf("alloc %d \n",n);  
  *dest = (char*)  malloc( n );
  strncpy( *dest , src + 1 , n );  
  *((*dest)+(n-1)) = 0;
  //debugf("- sscanStrQuotted \"%s\"\n",*dest);
  return e+1;
}

char* skipafter(char* src, char* pat) 
{
  char* str=0;
  //debugf("+ skipafter [%s]@[%s]\n",pat,src);
  //assert(src);
  if ( src == 0 ) return 0;
  str=strstr(src, pat);
  if (str) str+=strlen(pat);
  //debug(str);
  return str;
}
  

int fgets_strcmp_token(FILE* f, char* buf) 
{
  char* str=0;
  int i=0;
  *buf=0;
  //assert(buf); assert(f);
  //debug("+ fgets_strcmp_token");
  do {
    str = fgets(buf, BSIZE,f);
    //debug(buf);
    //assert(str); 
    if ( str == 0 ) return -1;
    //buf [ strlen(buf) - 2 ] = 0;
    //debugf("+ [%s]\n",buf);
    for (i=0; (i<nkey) || (str !=0 ) ;i++) {
      //debugf("? k[%i]=%s\n",i,keyword[i]);
      str = strstr( buf, keyword[i] );
      if (str ) return i;
    }
  } while( str == 0 );
  //assert(str);
  debug("- fgets_strcmp_token");
  return -2;
}

int fgets_tag(FILE* f, char* buf, char** strarg) 
{
  int t=0;
  char * str = 0;
  
  //  debug("+fgets_tag");
  //assert(f); assert(buf);  assert(strarg);
  t = fgets_strcmp_token(f, buf);
  //assert(t >= 0);
  //  debug(buf);
  //debug(keyword[t]);
  if (t >=0 ) str = skipafter( buf, keyword[t] );
  else { str = 0; }
  //debug(str);
  *strarg = str;
  //debug("-fgets_tag");
  return t;
}


int loadScene3dsAscii(char* filename, Scene* s) 
{
  FILE *f = 0;
  int t=0;
  //assert( filename); assert(s);
  //debugf("+loadScene3dsAscii \"%s\"\n",filename);
  if( !(f=fopen(filename, "ra")) ) {
    return errorIO(f,-1,"Err/IO/ Need a 3DS ascii file");
  }
  t = readScene3dsAscii( f, s);
  assert(t>0);
  fclose( f );
  //debugf("-loadScene3dsAscii (%d)\n",t);
  return t;
  
}

int readScene3dsAscii(FILE* f, Scene* s)
{
  char buf[BSIZE];
  int a,b,c=0;
  float x,y,z,u,v=0;
  unsigned int i=0;
  int t=0;
  char tmpc=0;
  char * str= NULL;
  unsigned int nv = 0;
  unsigned int nf =0;
  Object *o = NULL;
  //debug("+ readScene3dsAscii" );  
  //assert(f);  assert(s);  
  buf[0]= 0;
  str = & tmpc;
  initScene(s);
  t = fgets_tag(f,buf,&str);  
  while ( t >=0 ) {
    //debug(buf);
    switch(t) {
    case tag(trimesh):
      //debugf(".read trimesh %d\n",s->no);
      s->vo =(Object*)  realloc( s->vo, ++(s->no) * sizeof(Object) );
      
      o = & (s->vo)[ (s->no) - 1];
      sscanf( str, "%s %d %s %d", buf, &nv, buf, &nf );
      //debugf( "Vertices=%d; Faces=%d;\n",  nv , nf );
      
      o->vv  = (Vertex *) calloc( nv ,sizeof(Vertex) );
      o->vf = (Face *) calloc( nf, sizeof(Face) );
      o->nv = nv; 
      o->nf = nf;
      //debugf("read vertexes (%d)\n", o->nv);
      t = fgets_tag(f,buf,&str);
    case tag(vertexlist):
      //debugf("read vertext list %d\n",nv);
      //debug(str);
      for( i=0; i<nv; i++ ) { //Vertex 0:  X:-1.0	Y:1.0	Z:1.0
        t = fgets_tag(f,buf,&str);
        //assert ( t == VERTEX);
        str = skipafter( str, "X:"); assert(str); 
        sscanf(str,"%f",&x);
        str = skipafter( str, "Y:"); assert(str); 
        sscanf(str,"%f",&y);
        str = skipafter( str, "Z:"); assert(str);
        sscanf(str,"%f",&z);
        //debug(".texture");
        u = v = 0;
        str = skipafter( str, "U:"); 
        if (str) sscanf(str,"%f",&u);
        str = skipafter( str, "V:"); 
        if (str) sscanf(str,"%f",&v);
                
        //assert3gl( x , -BOUND , +BOUND );
        //assert3gl( y , -BOUND , +BOUND );
        //assert3gl( z , -BOUND , +BOUND );
        //assert3gl( u , 0 , 1);
        //assert3gl( v , 0 , 1);
        o->vv[i].x = x; o->vv[i].y = y; o->vv[i].z = z;
        o->vv[i].u = u; o->vv[i].v = v;
        //printV( o->vv[i] );
      }
      t = fgets_tag(f,buf,&str);  
    case tag(facelist):
      //debugf("read faces %d\n",nf);
      t = fgets_tag(f,buf,&str);
      for( i=0; i<nf; i++ ) { 
        //debugf("face[%d]=%s\n",i,buf);
         //str= skipafter( str, ":");
        //debug(str);
        initFace( & ( o->vf[i] ));
        //assert ( t == FACE);
        //assert(str);
        str = skipafter( str, "A:"); assert(str); 
        sscanf(str,"%d",&a);
        str = skipafter( str, "B:"); assert(str); 
        sscanf(str,"%d",&b);
        str = skipafter( str, "C:"); assert(str); 
        sscanf(str,"%d",&c);
        //debugf("face[%d]={ %d, %d, %d }; \n",i,a,b,c);
        o->vf[i].a = a; o->vf[i].b = b; o->vf[i].c = c;
        //assert3gl( a , 0 , o->nf);
        //assert3gl( b , 0 , o->nf);
        //assert3gl( c , 0 , o->nf);
        //

        t = fgets_tag(f,buf,&str);
        //debug(str);
        
        switch ( t ) {
        case tag(material) :
          str= skipafter( str, ":");
          sscanStrQuoted(str, & (o->vf[i].material) );
          //debugf(".-Material[%d]=\"%s\"\n",i, o->vf[i].material );
          t = fgets_tag(f,buf,&str);
          //debugf("%d=%s\n",t,buf);
        case tag(smoothing):
          //assert( t == SMOOTHING); // ??? I dont understand 
          
          //debugf("%d=%s\n",t,buf);
          switch ( t ) {
            case tag(smoothing):
              // debugf(".? %d Smoothing=\"%s\",\n",t,buf);
              str= skipafter( str, ":");
              sscanf(str,"%d", &t);
              o->vf[i].smoothing = t;
          //t = fgets_strcmp_token(f,buf); str = buf;
              t = fgets_tag(f,buf,&str);
              //debug(buf);
          }
          
        }
      }
    }
  }
  //debugf( "no=%d\n",s->no);
  //debugf("read v=(%d) f=(%d)\n", o->nv, o->nf);
  
  //printObject(o);
  //fprintObject(stdout, o);
  //debugf("- readScene3dsAscii (%d)\n",s->no);  
  return s->no;
}

int fprintScene(FILE* f, Scene const * const in) 
{
  int i=0,t=0;
  fprintf(f,"Ambiant light color: Red=%f, Green=%f Blue=%f\n",
          in->lightColorAmbiant.x, in->lightColorAmbiant.y, 
          in->lightColorAmbiant.z
          );
  
  for(i=0; i < in->no; i++) {
    t = fprintObject( f, & (in->vo[i]) );
  }
  return t;
}


int saveScene(Scene const * const in, char const * const filename)
{
  int t=0;
  unsigned int i=0; FILE* f=0;
  
  f=fopen( filename, "wt");
  if ( ! f ) return (int) f;
  fprintScene(f, in);
  fclose(f);
  return t;
}


int saveObject(Object const * const o, char const * const filename)
{
  int t=0;
  FILE* f=0;
  //debugf("+saveObject %s\n",filename);
  f=fopen( filename, "wt");
  if ( ! f ) return (int) f;
  t = fprintObject( f, o);
  fclose(f);
  //debug("-saveObject");
  return t;
  
  
}

int fprintObject(FILE* f, Object const * const  o)
{
  unsigned int i=0;
  fprintf(f,"Named object: \"");
  if ( o->name) fprintf(f,"%s",o->name);
  else
    fprintf(f,"%dV_%dF_@_www.rzr.online.fr", o->nv, o->nf);
  
  fprintf(f,"\"\r\n");
  fprintf(f,"\r\nTri-mesh, Vertices: %d Faces: %d\r\n",o->nv, o->nf);
  fprintf(f,"\r\nVertex list\r\n");
  
  for(i=0;i<o->nv;i++) {
    fprintf( f,"Vertex %d: X: %f Y: %f Z: %f ",i,
	     o->vv[i].x,  o->vv[i].y, o->vv[i].z);
    fprintf( f,"\t U: %f V: %f \r\n", o->vv[i].u,  o->vv[i].v);
  }
  fprintf(f,"\r\nFace list\r\n");
  for(i=0;i<o->nf;i++) {
    fprintf( f,"Face %d: A:%d B:%d C:%d",i,
             o->vf[i].a, o->vf[i].b,  o->vf[i].c
             );
    fprintf(f,"\t AB:1 BC:1 CA:1 \r\n"); //!!!
    if( o->vf[i].material ) {
      fprintf(f, "Material: \"%s\"\r\n",o->vf[i].material);
      fprintf(f, "Smoothing:1\r\n");
    }
    
    //fprintf( f, "Smoothing: \"%d\"\r\n",o->vf[i].smoothing);
  }
  return 1;
}



int parse3dsAscii(FILE* f, Object* o) 
{
  char buf[BSIZE];
  int a,b,c=0;
  float x,y,z=0;
  int i=0;
  int t=0;
  char tmpc;
  int nv = 0;
  int nf =0;
  char * str = NULL;
  //debug("+ parse3dsAscii");  
  exit(0);
  
  buf[0]= 0;
  str = & tmpc;
  //debug("skip to 1st object");
  t = fgets_tag(f,buf,&str);  
  while ( t >=0 ) {
    switch(t) {
    case trimesh:
      //debug("read trimesh");
      sscanf( str, "%s %d %s %d", buf, &nv, buf, &nf );
      //debugf( "Vertices=%d; Faces=%d;\n",  nv , nf );
      
      o->vv  = (Vertex *) calloc( nv ,sizeof(Vertex) );
      o->vf = (Face *) calloc( nf, sizeof(Face) );
      o->nv = nv; 
      o->nf = nf;
      //debugf("read vertexes (%d)\n", o->nv);
      t = fgets_tag(f,buf,&str);
    case vertexlist:
      //debugf("read vertext list %d\n",nv);
      //debug(str);
      for( i=0; i<nv; i++ ) { //Vertex 0:  X:-1.0	Y:1.0	Z:1.0
        t = fgets_tag(f,buf,&str);
        assert ( t == vertex);
        str = skipafter( str, "X:"); assert(str); 
        sscanf(str,"%f",&x);
        str = skipafter( str, "Y:"); assert(str); 
        sscanf(str,"%f",&y);
        str = skipafter( str, "Z:"); assert(str); 
        sscanf(str,"%f",&z);
        //debugf("vertex[%d]={ %f, %f, %f };\n",i,x,y,z);
        //assert3gl( x , -BOUND , +BOUND );
        // assert3gl( y , -BOUND , +BOUND ); assert3gl( z , -BOUND , +BOUND );
        o->vv[i].x = x; o->vv[i].y = y; o->vv[i].z = z;
        //exit(0);
      }
      t = fgets_tag(f,buf,&str);  
    case facelist:
      //debugf("read faces %d\n",nf);
      t = fgets_tag(f,buf,&str);
      for( i=0; i<nf; i++ ) { //Vertex 0:  X:-1.0	Y:1.0	Z:1.0
        //str= skipafter( str, ":");
        //debug(str);
        //assert ( t == FACE);
        //assert(str);
        str = skipafter( str, "A:"); assert(str); 
        sscanf(str,"%d",&a);
        str = skipafter( str, "B:"); assert(str); 
        sscanf(str,"%d",&b);
        str = skipafter( str, "C:"); assert(str); 
        sscanf(str,"%d",&c);
        //debugf("face[%d]={ %d, %d, %d }; \n",i,a,b,c);
        o->vf[i].a = a; o->vf[i].b = b; o->vf[i].c = c;
        //assert3gl( a , 0 , o->nf);
        //assert3gl( b , 0 , o->nf);        assert3gl( c , 0 , o->nf);
        t = fgets_tag(f,buf,&str);
        //debug(str);
        switch ( t ) {
        case material :
          str= skipafter( str, ":");
          sscanStrQuoted(str, & (o->vf[i].material) );
          //debugf(".-Material[%d]=\"%s\"\n",i, o->vf[i].material );
          t = fgets_tag(f,buf,&str);
          //debugf("%d=%s\n",t,buf);
        case smoothing:
          //debug(str);
          //debug(pat);
          str= skipafter( str, ":");
          sscanf(str,"%d", &t);
          o->vf[i].smoothing = t;
          t = fgets_strcmp_token(f,buf); str = buf;
        }
      }
      
    }
  }
  //debugf("read v=(%d) f=(%d)\n", o->nv, o->nf);
  //debug("- parse3dsAscii");  
  return 0;
}

int load3dsAscii(char* filename, Object* o)
{ 
  FILE *f = 0;
  int t=0;
  //debugf("+read parse3ds %s\n",filename );
  if( !(f=fopen(filename, "ra")) ) {
    perror("Err/IO/ Need a 3DS ascii file \n");
    return(-1);
  }
  t = parse3dsAscii( f, o);
  fclose( f );
  //debugf("+read parse3ds %d\n",t);
  return t;
}


/*
  mesh {
    triangle {
      <-10, 10, -10>, <10, 10, -10>, <10, 10, 10>
      texture { Red }
    }
    triangle {


mesh2 {
   vertex_vectors {
      24, 
      <0,0,0>, <0.5,0,0>, <0.5,0.5,0>, //1
      <0.5,0,0>, <1,0,0>, <0.5,0.5,0>, //2
      <1,0,0>, <1,0.5,0>, <0.5,0.5,0>, //3
      <1,0.5,0>, <1,1,0>, <0.5,0.5,0>, //4
      <1,1,0>, <0.5,1,0>, <0.5,0.5,0>, //5
      <0.5,1,0>, <0,1,0>, <0.5,0.5,0>, //6
      <0,1,0>, <0,0.5,0>, <0.5,0.5,0>, //7
      <0,0.5,0>, <0,0,0>, <0.5,0.5,0>  //8
   }
   face_indices {
      8, 
      <0,1,2>,    <3,4,5>,       //1 2
      <6,7,8>,    <9,10,11>,     //3 4
      <12,13,14>, <15,16,17>,    //5 6
      <18,19,20>, <21,22,23>     //7 8
   }
   pigment {rgb 1}
}


*/
//#include <stdio.h>
//#include "types3d.h"




#endif // inclusion
