/* #ident "$Id: trace3d.c,v 1.1.1.1 2004/03/30 12:09:28 rzr Exp $" */
/**
 * @author: www.Philippe.COVAL.online.fr - Rev: $Author: rzr $ 
 * Copyright: GPL @ http://rzr.online.fr/licence.htm
 **/

#ifndef trace3d_c_
#define trace3d_c_

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "trace3d.h"
#include "include.h" //SQRT2


Vertex addVV( Vertex a, Vertex b)
{
  a.x += b.x;
  a.y += b.y;
  a.z += b.z;
  return a;
}

Vertex subVV( Vertex a, Vertex b)
{
  a.x -= b.x;
  a.y -= b.y;
  a.z -= b.z;
  return a;
}


Vertex multVf(Vertex a, float arg)
{
  //debugf("+multVf %f *" ,arg);  print(a);
  a.x *= arg;
  a.y *= arg;
  a.z *= arg;
  //print(r);
  return a;
}

Vertex multVV(Vertex a, Vertex b)
{
  a.x *= b.x;
  a.y *= b.y;
  a.z *= b.z;
  return a;
}

Vertex minVV(Vertex a, Vertex b)
{
  a.x = min2(a.x, b.x);
  a.y = min2(a.y, b.y);
  a.z = min2(a.z, b.z);
  return a;
  
}

Vertex maxVV(Vertex a, Vertex b)
{
  a.x = max2(a.x, b.x);
  a.y = max2(a.y, b.y);
  a.z = max2(a.z, b.z);
  return a;
}


float maxV(Vertex a) 
{
  return max3(a.x, a.y, a.z);
}

float minV(Vertex a) 
{
  return min3(a.x, a.y, a.z);
}



void multMV(float* r0, float* r1, float r2, 
            float v0, float v1, float v2, 
            float m00, float m01, float m02,
            float m10, float m11, float m12,
            float m20, float m21, float m22)
{
  //r0 = v0 * m00 + 
  exit(0);
}

//-----------------------------------------------------------------------------
void stats( Vertex* vv, int nVertex,
            Face* vFace, int nFace,
            Vertex* lo, Vertex* hi, Vertex* mid, Vertex* center) 
{
  //debug("+stats");
  Vertex size;
  float f= 0;
  int i=0;
  float x,y,z;
  initVertex(center);
  initVertex(hi);
  initVertex(lo);
   
  *lo = addVV(*lo, vv[0]);
  *hi = addVV(*hi, vv[0]);


  f = 1./ (float) nVertex;
  
  printf("nVertex=%d; nFace=%d; vpf=%f; \n", nVertex, nFace,
         (float) nVertex / (float) nFace);

  for(i=0; i < nVertex ; i++) {
    x =  vv[i].x;
    y =  vv[i].y;
    z =  vv[i].z;

    assert3gl( x , -1000, +1000);
    assert3gl( y , -1000, +1000);
    assert3gl( z , -1000, +1000);

    if ( vv[i].x < (*lo).x) (*lo).x = vv[i].x;
    if ( vv[i].y < (*lo).y) (*lo).y = vv[i].y;
    if ( vv[i].z < (*lo).z) (*lo).z = vv[i].z;

    if ( vv[i].x > (*hi).x) (*hi).x = vv[i].x;
    if ( vv[i].y > (*hi).y) (*hi).y = vv[i].y;
    if ( vv[i].z > (*hi).z) (*hi).z = vv[i].z;
    
    *center = ( addVV( *center, multVf ( vv[i], f)));
  }
  

  printf("hi=");
  printV(*hi);

  printf("lo=");
  printV(*lo);
  
  *mid = multVf( addVV(*lo,*hi) , 0.5);
  printf("mid=");
  printV(*mid);
  
  size = subVV( *hi, *lo);
  printf("size=");
  printV(size);

  printf("center=");
  printV( *center);
  //debug("-stats");  
}

void placeObject( Object *o, float x, float y, float z)
{
  Vertex lo,hi,med,center;
  statsObject( o,  &lo, &hi, &med, &center);
  translateObject( o, x - lo.x , y - lo.y  , z - lo.z);
}

void rotateAxisObject( Object *o, float x,float y, float z,float a)
{
  //debug("use quaternions");
  exit(-1);
}

void rotateObject( Object *o, float x,float y, float z)
{
  Vertex rot; rot.x=x; rot.y=y; rot.z=z;
  rotate(&o->vv, o->nv, rot);
}
void translateObject( Object *o, float x,float y, float z)
{
  Vertex rot; rot.x=x; rot.y=y; rot.z=z;
  translate(&o->vv, o->nv, rot);
}

void transformScene(Scene* in , Vertex const * const t, 
                    Vertex const * const r, Vertex const * const s) 
{
  unsigned int i=0;
  for(i=0;i<in->no ;i++) {
    translateObject( &(in->vo)[i], t->x, t->y, t->z);
    rotateObject( &(in->vo)[i], r->x, r->y, r->z );
    scaleObject(  &(in->vo)[i], s->x, s->y, s->z );
  }
}


void rotate( Vertex** vv, int nVertex, Vertex rot)
{
  float S , C , x,z;
  int i;
  //debugf("+rotate %f\n",rot.x);
  S = sin( rot.x );  C = cos( rot.x );
  //debugf("%f %f \n",S,C);
  for( i=0; i<nVertex; i++ ) {
    z = (*vv)[i].z;
    (*vv)[i].z = (*vv)[ i ].y * (- S) + z * C;
    (*vv)[i].y = (*vv)[ i ].y * C + z * S;
  } 
  S = sin( rot.y );  C = cos( rot.y );
  for( i=0; i<nVertex; i++ ) {
    x = (*vv)[i].x;
    (*vv)[i].x =  x * ( C ) + (*vv)[i].z * ( -S );
    (*vv)[i].z =  x * ( S ) + (*vv)[i].z * (  C );
  }
  S = sin( rot.z );  C = cos( rot.z );
  for( i=0; i<nVertex; i++ ) {
    x = (*vv)[i].x;
    (*vv)[i].x = x * (  C ) + (*vv)[i].y * (  S );
    (*vv)[i].y = x * ( -S ) + (*vv)[i].y * (  C );
  }
  //debug("-rotate");
}

void translate( Vertex** vv, int nVertex,
                Vertex off)
{
  int i=0;
  //debug("+translate");
  printV(off);
  for(i=0;i<nVertex;i++)    (*vv)[i] = addVV( (*vv)[i], off);
  //debug("-translate");
}

void printObject( Object* o) 
{
  Vertex lo,hi,med,center;
  //debug("+printObject");
  debugf("nv=%d; nf=%d\n",o->nv,o->nf);
  statsObject( o,  &lo, &hi, &med, &center);
}

void scaleObjectInSphere( Object* o, float r)
{
  Vertex lo,hi,med,center;
  statsObject( o,  &lo, &hi, &med, &center);
  scaleInSphere( &(o->vv), o->nv, r,lo,hi);
}


void scaleSceneInSphere( Scene* o, float r)
{
  int i=0;
  Vertex lo,hi,med,center;
  statsSceneBox( o,  &lo, &hi);
  for(i=0;i < o->no; i++)
    scaleObjectInSphere( &(o->vo[i]) , r);
  
}

void statsObject( Object* obj,
                  Vertex* lo, Vertex* hi, Vertex* mid, Vertex* center) 
{
  stats( obj->vv, obj->nv, obj->vf, obj->nf, lo, hi, mid ,center); 
}

void centerObject(Object* o)
{
  Vertex lo,hi,med,center;
  statsObject( o,  &lo, &hi, &med, &center);
  med = multVf(med,-1);
  translate( &(o->vv), o->nv , med );
}
void scaleInSphere( Vertex** v, int n, float r, Vertex lo, Vertex hi)
                    
{
  Vertex arg;
  //debugf("+scaleInSphere %f\n",r);
  arg.x =  hi.x - lo.x;
  arg.y =  hi.y - lo.y;
  arg.z =  hi.z - lo.z;
  r =  r / ( (SQRT2/2) *  max3( abs(arg.x), abs(arg.y), abs(arg.z) ) ) ;
  //debugf("s=%f\n", r);
  arg.y = arg.z = arg.x = r;
  scale(v, n , arg);
  //debug("-scaleInSphere");
}
void scale( Vertex** vv, int nVertex, Vertex arg)
{
  int i=0;
  //debug("+scale");
  printV(arg);
  for(i=0;i<nVertex;i++)    (*vv)[i] = multVV( (*vv)[i], arg);
  //debug("-scale");
}
void scaleObject(Object* o, float x, float y, float z)
{
  Vertex rot; rot.x=x; rot.y=y; rot.z=z;
  scale(&o->vv, o->nv, rot);
}


void statsSceneBox(Scene* s, Vertex* lo, Vertex* hi)
{
  unsigned int i;
  Vertex tcenter,tlo,thi,tmed,center;
  Object* o;
  //debug("+ statsSceneBox");
  o = &(s->vo[0]);
  statsObject( o , lo, hi, &tmed, &center);

  for (i=1; i< s->no; i++) {
    o = &(s->vo[i]);
    statsObject( o , &tlo, &thi, &tmed, &tcenter);
    *lo = minVV(*lo, tlo);
    *hi = maxVV(*hi, thi);
  }
  //printV(*lo);
  //printV(*hi);
  //debug("- statsSceneBox");
}

//-----------------------------------------------------------------------------
/*
  <pre>

  screen: right hand direct

  y(g)
  | 
  *----x(r)
  / 
  Z(B)

  </pre>
*/


void project( Point** vPoint,
              Vertex* vv, int nVertex,
              Face* vFace, int nFace,
              Vertex lo, Vertex hi,
              int w, int h) 
{
    
  int offx = 0 ;  int offy = 0 ;
  float scalex = 0;
  float scaley = 0;
  int i=0;
  float x,y,z=0;
  float e=3.0; //eye  
  float f=1.;
  //debugf("+projection %d\n",nVertex);
  
  *vPoint = (Point*) calloc( nVertex, sizeof( Point) );
  printV(lo);
  printV(hi);
  
  scalex = (float) ( w/2)  /  ( hi.x - lo.x ); // 1/2 
  scaley = (float) ( h/2)  /  ( hi.y - lo.y ); 
  scalex = min2(scalex, scaley) / 2.; //!!!
  scaley = scalex;
  offx = w / 2;
  offy = h / 2;
  
  //debugf("project off={%d,%d}; scale={%f,%f};\n",offx, offy, scalex, scaley);
         
  
  for( i=0; i<nVertex; i++ ) {
    x = vv[i].x ;
    y = vv[i].y ;
    z = vv[i].z ; //z = abs(z); //assert2le (  0, z  );
    //debugf("z=%f\n",z);
    // paralel
    f = 1.;
    // perspect
    f =  maxV(hi) - minV(lo) ;
    //debugf ( "eye=%f\n",f);
    f =  1. -  ( 1. /  ( SQRT2 * f   + z )) ;
    //debugf ( "f=%f\n",f);
    assert2ge( f, 0);
    (*vPoint)[i].x = float2int ( x  * f  * scalex) + offx ;
    (*vPoint)[i].y = float2int ( y  * f  * - scaley) + offy ;

    // printPoint( (*vPoint)[i] );
    assert2le (  0, (*vPoint)[i].x  );
    assert2lt (  (*vPoint)[i].x , w );
    assert2le (  0, (*vPoint)[i].y  );
    assert2lt (  (*vPoint)[i].y , h );
    //(*vPoint)[i].x = max2( (*vPoint)[i].x) , x ) ; 
    //(*vPoint)[i].x  = min2 ( *vPoint)[i].x  , w);
  }
  //exit(0);
  debug("-projection");
}

#endif // inclusion
